"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var chalk_1 = __importDefault(require("chalk"));
var path_1 = require("path");
var SEPARATOR = '\n';
var WARNING = 'WARNING';
var ERROR = 'ERROR';
var isError = function (severity) { return severity === 2; };
var pad = function (str, minLen, maxLen) { return (maxLen <= minLen
    ? str
    : "" + str + ' '.repeat(maxLen - minLen)); };
var buildPath = function (filePath, line, column) {
    var path = !filePath.includes('/') && !filePath.includes('\\') ? "." + path_1.sep + filePath : filePath;
    return path + ":" + line + (column ? ":" + column : '');
};
var getErrorType = function (severity, isRaw) {
    if (isRaw === void 0) { isRaw = true; }
    var type = isRaw
        ? isError(severity) ? ERROR : WARNING
        : isError(severity) ? chalk_1.default.red(ERROR) : chalk_1.default.blue(WARNING);
    return type + ": ";
};
var buildMessage = function (_a, maxPathLen, maxRuleLen) {
    var errorType = _a.errorType, message = _a.message, rule = _a.rule, path = _a.path;
    return ("" + errorType + pad(path.filePath, path.len, maxPathLen) + " " + pad(rule.name, rule.len, maxRuleLen) + " " + message);
};
var buildOutput = function (result) {
    var initialData = {
        maxRuleLen: 0,
        maxPathLen: 0,
        messages: [],
    };
    var _a = result.messages.reduce(function (_a, _b) {
        var maxRuleLen = _a.maxRuleLen, maxPathLen = _a.maxPathLen, messages = _a.messages;
        var severity = _b.severity, message = _b.message, ruleId = _b.ruleId, column = _b.column, line = _b.line;
        var errorType = getErrorType(severity);
        var path = buildPath(result.filePath, line, column);
        var ruleName = ruleId ? ruleId : '';
        var ruleLen = ruleName.length;
        var pathLen = errorType.length + path.length;
        return {
            maxRuleLen: Math.max(maxRuleLen, ruleLen),
            maxPathLen: Math.max(maxPathLen, pathLen),
            messages: messages.concat([{
                    errorType: getErrorType(severity, false),
                    message: chalk_1.default.yellow(message),
                    rule: { name: chalk_1.default.grey(ruleName), len: ruleLen },
                    path: { filePath: path, len: pathLen },
                }]),
        };
    }, initialData), maxRuleLen = _a.maxRuleLen, maxPathLen = _a.maxPathLen, messages = _a.messages;
    return [
        chalk_1.default.white(result.filePath)
    ].concat(messages.map(function (message) { return buildMessage(message, maxPathLen, maxRuleLen); })).join(SEPARATOR);
};
module.exports = function (results) { return (results
    .filter(function (result) { return result.errorCount || result.warningCount; })
    .map(buildOutput).join(SEPARATOR)); };
